#ifndef	_DGA_DGACMAPSTR_H
#define	_DGA_DGACMAPSTR_H

#pragma ident	"@(#)dgacmapstr.h	1.6	94/11/02 SMI"

/*
 *   ----------------------------------------------------------------- 
 *          Copyright (C) 1986,1991  Sun Microsystems, Inc
 *                      All rights reserved. 
 *            Notice of copyright on this source code 
 *            product does not indicate publication. 
 *   
 *                    RESTRICTED RIGHTS LEGEND: 
 *   Use, duplication, or disclosure by the Government is subject 
 *   to restrictions as set forth in subparagraph (c)(1)(ii) of 
 *   the Rights in Technical Data and Computer Software clause at 
 *   DFARS 52.227-7013 and in similar clauses in the FAR and NASA 
 *   FAR Supplement. 
 *   ----------------------------------------------------------------- 
 */


/* 
 *
 * Colormap grabber header file.
 *
 *
 * Macros defined here:
 *
 *  Dga_cm_lock(infop)
 *  Dga_cm_unlock(infop)
 *	lock and unlock colormap info page.  'infop' is the Grabbedcmap
 *	pointer returned by cmapgrab_grab()
 *
 *  Dga_cmapgrab_cookie(infop)
 *	returns the colormap info page cookie used to map the info page.
 *	this is the value that should be returned to the requesting
 *	client.
 *
 ****/

#include <sys/types.h>
#include "X.h"
#include "pixmapstr.h"
#include "windowstr.h"
#include "colormapst.h"
#include "colormap.h"


/* Tell client how to load colormaps */

typedef	enum {
    SUNWINDOWS_IOCTL=0,	/* use /dev/winxx SunWindows device driver */
    HW_DEVICE_IOCTL=1,	/* use graphics hardware device driver */
    HW_DEVICE_DIRECT=2	/* go straight to HW */
} cmap_load_method ;


#define	CMMAGIC	0x434d4150	/* "CMAP" */
#define	CMVERS	2

#define	MAX_CM_DEVNAME	40	/* better not have long device names */
#define DGA_CM_DEV_INFO_SZ 132

/* info in shared memory */
typedef	struct _grabbedcmap
{
    /* version 0 info starts here */
	long	cm_magic ;	/* "CMAP" = 0x434d4150 ("PAMC" on 386i) */
	long	cm_version ;	/* 2 */
	int	cm_count ;	/* change count */
	struct _grabbedcmap *cm_ptr ;	/* self-reference */
	u_long	cm_cmap ;	/* X11 colormap ID */

	u_long	cm_cookie ;	/* lock-page cookie */

	/* server-specific info, not meaningful to client */
	int	cm_sinfofd ;	/* fd of this file */
	int	cm_sdevfd ;	/* fd of framebuffer */
	u_long	*cm_slockp ;	/* pointer to lock page */
	u_long	*cm_sunlockp ;	/* pointer to unlock page */
	int	cm_slockcnt ;	/* server lock count, for nesting */
	int	cm_clientcnt ;	/* number of clients grabbing this cmap */
	int	cm_filesuffix ;	/* internal bookkeeping */
	WindowPtr	cm_shras ;	/* internal raster handle */

	char	cm_devname[MAX_CM_DEVNAME] ;		/* fb */
	char	cm_lockdevname[MAX_CM_DEVNAME] ;	/* lockpages */
	int	cm_default ;	/* is it the default cmap for screen */
	int	cm_installed ;	/* flag: colormap is installed in HW */
	cmap_load_method cm_load_method ;

	/* server's /dev/winxx used to set colormaps, default colormap
	   name ("xnews_default") */
	char	cm_sunwindows_def_devname[MAX_CM_DEVNAME] ;
	char	cm_sunwindows_def_cmapname[MAX_CM_DEVNAME] ;
	int	cm_sunwindows_def_cmapsize ;

	/* server's /dev/winxx used to set colormaps, installed colormap
	   name ("xnews_installed") */
	char	cm_sunwindows_inst_devname[MAX_CM_DEVNAME] ;
	char	cm_sunwindows_inst_cmapname[MAX_CM_DEVNAME] ;
	int	cm_sunwindows_inst_cmapsize ;
	

	int	cm_fbtype ;	/* reserved */
	int	cm_reallen ;	/* colormap hardware size */
	int	cm_start ;	/* first valid location */
	int	cm_len ;	/* number of valid locations */
	int	cm_cmapnum ;	/* obsolete [cmap num on FB's with MHC] */
	int	cm_ioctlbits ;	/* obsolete [high-order bits for FBIOPUTCMAP]*/
	int	cm_devtype ;	/* device type, see sh_scr_dev.h */
	int	cm_clientype;	/* 0 for old-style clients, 1 for new */
	int	cm_reserved[4] ; /* reserved */

	long	cm_shadowofs ;	/* byte offset relative to cm_magic of
				   the start of the shadow colormap */

    /* version 1 info starts here */
	int	cm_scount ;	/* server's last remembered count */
	int	cm_locktype ;	/* using /dev/winlock?, see below */

    /* version 2 info starts here */
	u_int   device_offset;	/* offset to device specific section */

	/* Do NOT add any fields below dev_info */
	u_char  dev_info[DGA_CM_DEV_INFO_SZ];/* device specific information */
} Grabbedcmap ;


	/* followed by three arrays of u_shorts, length cm_len, for
	   red, green and blue and one array of bytes which are
	   the change flags for their respective colors:
	   
		rr
		rr
		rr
		:
		gg
		gg
		gg
		:
		bb
		bb
		bb
		:
		f
		f
		f
		:
		
	    Clients set a flag to non-zero to indicate that the color
	    has been changed.  The server clears these when the changes
	    have been seen.
	  */


#define	CM_LOCKDEV	0	/* use GRABPAGEALLOC from device */
#define	CM_WINLOCK	1	/* use winlock functions */

#ifdef CLIENTSIDE
/* Client-side info that can't be stored in shared memory because
   there may be more than one client. */


typedef	struct cmap_grab
{
	Grabbedcmap	*cm_info ;	/* pointer to shared memory */
	struct cmap_grab *cm_next ;	/* for client use */
	caddr_t		cm_client ;	/* for client use */
	int		cm_count ;	/* last recorded change count */
	int		cm_cinfofd ;	/* fd of shared memory file */
	int		cm_cdevfd ;	/* fd of framebuffer */
	int		cm_clockfd ;	/* fd of lock device (if not fb) */
	int		cm_csunwindows_def_fd ; /* client fd for server's
					   SunWindows default cmap window */
	int		cm_csunwindows_inst_fd ; /* client fd for server's
					   SunWindows installed cmap window */
	u_long		*cm_clockp ;	/* pointer to lock page */
	u_long		*cm_cunlockp ;	/* pointer to unlock page */
	int		cm_lockcount ;	/* lock count for nested locks */
	int		cm_use_new_ioctl ; /* use new hardware ioctl? */
} Cmap_Grab ;
#endif /* CLIENTSIDE */



#define CM_PAGESZ       (8*1024)
#define CM_LOCK(x)      (*(x)) = 1
#define CM_UNLOCK(x)    (*(x)) = 0


#define	DgaCmapCookie(infop)	((infop)->cm_cookie)

#define	DgaCmapFilesuffix(infop)	((infop)->cm_filesuffix)

#define	DgaCmapGrabLock(infop)		\
    do {				\
    if( (infop)->cm_slockcnt++ == 0 )	\
	*(infop)->cm_slockp = 1 ;	\
    } while (0)

#define	DgaCmapGrabUnlock(infop)		\
    do {				\
    if( --(infop)->cm_slockcnt <= 0 )	\
	*(infop)->cm_sunlockp = 0 ;	\
    } while (0)


#define	DgaCmapGrabInstall(infop)	\
	do {				\
	  infop->cm_installed = 1 ;	\
	  ++infop->cm_count ;		\
	} while(0) ;

#define	DgaCmapGrabUninstall(infop)	\
	do {				\
	  infop->cm_installed = 0 ;	\
	  ++infop->cm_count ;		\
	} while(0) ;


typedef struct {
	Grabbedcmap	*pDgaCmapGrabInfop;

} DgaCmapRec, *DgaCmapPtr;
#ifdef COMMENT
#define DgaGetCmapPriv(s) ((DgaCmapPtr)((s)->devPriv))
#define DgaSetupCmap(s) DgaCmapPtr	pDgaCmap = DgaGetCmapPriv(s)
#endif /* COMMENT */


#endif /* _DGA_DGACMAPSTR_H */
