#ifndef _SUNDPA_H
#define _SUNDPA_H
#pragma ident "@(#)sundpa.h	1.1 94/11/20   SMI" 

/*
 *   ----------------------------------------------------------------- 
 *          Copyright (C) 1994  Sun Microsystems, Inc
 *                      All rights reserved. 
 *            Notice of copyright on this source code 
 *            product does not indicate publication. 
 *   
 *                    RESTRICTED RIGHTS LEGEND: 
 *   Use, duplication, or disclosure by the Government is subject 
 *   to restrictions as set forth in subparagraph (c)(1)(ii) of 
 *   the Rights in Technical Data and Computer Software clause at 
 *   DFARS 52.227-7013 and in similar clauses in the FAR and NASA 
 *   FAR Supplement. 
 *   ----------------------------------------------------------------- 
 */

#include <dpa/sundpatype.h>

/*
 * Sun Direct Pixel Access Interface (DPA).
 *
 *	This interface enables a Screen handler to allow other moudules
 *	of the window server program to directly access the pixels of Drawables 
 *	managed by the handler.
 *
 *	The intent is to allow graphics extensions (DisplayPostScript, for
 *	example) to attain higher performance by often avoiding extra
 *	pixel copies.
 *
 *	The caller queries the handler to determine whether or not DPA
 *	will be allowed for a given Drawable. If the query funtion returns
 *	TRUE, the caller may then attempt to begin a direct access cycle.
 *
 *	If that succeeds the drawable parameters are returned by the handler and
 *	the caller is free to read and write the pixels.
 *
 *	When the caller has completed the operation it ends the cycle.
 *
 *	Note that since DPA is not available on all Screens and Drawables
 *	and access can be denied at any time, the caller must be prepared
 *	to fall back to standard ddx operations.
 *
 */


#ifdef DONTNEED
/*
 *  sunDPADirectAccess()
 *      Returns TRUE if the caller will be allowed use direct pixel access
 *	on a Drawable.
 *
 *	Note that this is a query function only. The caller
 * 	must use sunDPAStart() to actually begin a direct access cycle.
 */

extern Bool sunDPADirectAccess(DrawablePtr pDraw);

#endif



/*
 *  sunDPADirectAccess()
 *	Given two drawables, the Type of DPA allowed for the pair of drawables.
 */


extern sunDPAccessType sunDPADirectAccess(DrawablePtr pDraw1,
					  DrawablePtr pDraw2);

/*
 * sunDPAStart()
 *      Begins a direct pixel access cycle for a given drawable.
 *	Returns TRUE and sets Pixel memory parameters if direct access is
 *	allowed, FALSE otherwise.
 *
 *	Device dependent register state is set up so that the pixels can be
 *	accessed, correctly.
 *	
 *	This function may be called multiple times, with different drawables if
 *	sunDPADirectAccess2 returned TRUE for the Drawables.
 *
 *	Note: That it is possible that the query functions
 *	sunDPADirectAccess or sunDPADirectAccess2 will return TRUE
 *	for a given Drawable(s), but the handler might choose to deny access
 *	when the caller attempts to start a direct access cycle.
 * 	The caller must be prepared to handle this case.
 *
 *	When the caller completes it's operation, it must call sunDPAEnd() to
 *	end the cycle. Since the drawable is locked during the cycle for best
 *	system performance, DPA cycles should be kept short.
 */

extern Bool sunDPAStart(DrawablePtr pDraw, CARD8**p, int *bytesPerRow);

/*
 *  sunDPADirectAccessEnd()
 *      Ends a direct access cycle for a given Drawable.
 */          

extern void sunDPAEnd(DrawablePtr pDraw);

/*
 *  sunDPAFakePixmapsOK()
 *      Returns TRUE if the screen can handle fake pixmaps. That is, PixmapRec's
 *      that are not created through pScreen->CreatePixmap and thus who's data
 *	area points to randomly allocated system memory. If this function
 *	succeeds it also implys that the handler expects that the Fake Pixmap's
 *	devPrivate is a pointer to the pixmap data and devKind is the
 *	bytes per scanline (That is the same format that mfb & cfb expect).
 */

extern Bool sunDPAFakePixmapsOK(ScreenPtr pScreen);

/*
 *  sunDPADirectAccessDPS()
 *      Returns TRUE if DPS should use direct access to mark to the drawable,
 *      returns FALSE if caller should use ddx
 *
 *	This is a performance hint from the ddx handler to Display PostScript.
 */

extern Bool sunDPADirectAccessDPS(DrawablePtr pDraw);

#endif /* !defined(_SUNDPA_H) */
