#ifndef _SUNDPASCR_H
#define _SUNDPASCR_H

#pragma ident "@(#)sundpascr.h	1.1 94/11/20   SMI" 
/*
 *   ----------------------------------------------------------------- 
 *          Copyright (C) 1994  Sun Microsystems, Inc
 *                      All rights reserved. 
 *            Notice of copyright on this source code 
 *            product does not indicate publication. 
 *   
 *                    RESTRICTED RIGHTS LEGEND: 
 *   Use, duplication, or disclosure by the Government is subject 
 *   to restrictions as set forth in subparagraph (c)(1)(ii) of 
 *   the Rights in Technical Data and Computer Software clause at 
 *   DFARS 52.227-7013 and in similar clauses in the FAR and NASA 
 *   FAR Supplement. 
 *   ----------------------------------------------------------------- 
 */

/*
 * Sun Direct Pixel Access Interface.
 *
 *	This interface allows other parts of the window server to directly
 *	access pixels in Drawables managed by a DDX Screen handler.
 *	This allows extensions such as Display PostScript to attain higher
 *      performance by often avoiding expensive pixel copies.
 *
 *	A handler enables direct pixel access (DPA) by calling the function
 *	sunDPAScreenInit() during Screen initialization.
 *	If the handler does not call sunDPAScreenInit, DPA is disabled for
 *	all Drawables handled by the Screen.
 */

#include <X11/Xmd.h>
#include <pixmapstr.h>
#include <dpa/sundpatype.h>

/* List of predefined built-in DPA modes. See Description of below. */

typedef enum { 
	sunDPANone,
	sunDPACustom,
	sunDPAPixmap,
	sunDPAAllDrawables
} sunDPAMode;


typedef struct {
       sunDPAMode	 mode;
       sunDPAMode	 dpsMarkMode;
       sunDPAccessType	 (*directAccessOK)(DrawablePtr pDraw1,
					   DrawablePtr pDraw2);
       Bool		 (*directAccessDPS)(DrawablePtr pDraw);
       Bool		 (*directAccessStart)(DrawablePtr pDraw,
					      CARD8  **p, int *bytesPerRow);
       void		 (*directAccessEnd)(DrawablePtr);
       CARD32		 reserved[8];	/* these must be set to zero */
} sunDPAScreenRec;

/*
 * sunDPAScreenInit()
 *
 *	Enable Direct Pixel Access for a Screen.
 *	The structure passed to sunDPAScreenInit() is only needed for
 *	the duration of the function, so the struct may be freed after
 *	sunDPAScreenInit returns.
 */
extern void sunDPAScreenInit(ScreenPtr, sunDPAScreenRec *);

/*
 * sunDPAScreenRec
 *
 *	Data structure that describes a handler's DPA capabilities.
 *	A pointer to a sunDPAScreenRec is passed by the handler to
 *	sunDPAScreenInit().
 *
 *		mode
 *				The sunDPAMode for direct Access.
 *				If the mode is sunDPACustom, the function
 *				pointers directAccess and directAccessStart
 *				must point to the handler's functions.
 *				If the mode is any other value, the function
 *				pointers are not used. (See the description
 *				of the predefined modes below.)
 *
 *				If the mode is sunDPANone, dpsDPAMarkMode
 *				is ignored. (Since sunDPANone is the default
 *				mode, the handler need not call sunDPAScreenInit
 *				to disable DPA for a Screen.)
 *
 *		dpsDPAMarkMode
 *				DPA mode to be used by for DPS marking.
 *				If the mode is sunDPACustom, the function
 *				pointer directAccessDPS must be set to
 *				the handler's function.
 *
 *		directAccessOK				
 *				Pointer to a function that returns the 
 *				sunDPAccessType possible for two Drawables.
 *
 *				If the second Drawable is NULL, the caller
 *				is only requesting DPA for a single Drawable.
 *				Both pointers may point to the same Drawable.
 *
 *				The handler's directAccessOk function  should
 *				return
 *
 *				    sunDPANone      if DPA is not allowed
 *
 *				    sunDPABoth	    if DPA is ok to both 
 *					            Drawables simultaneously.
 *
 *				    sunDPAOne	    if DPA is ok for the first
 *						    but not the second. 
 *						    This value should also be
 *						    returned if DPA is OK for
 *						    the first Drawable and the
 *						    second Drawable is NULL.
 *
 *				    sunDPATwo	    if DPA is ok for the second
 *						    but not the first.
 *
 *				    sunDPAEitherNotBoth if DPA is ok for both
 *						    Drawables individually, just
 *						    not simultaneously.
 *
 *				The DPA user will call the function
 *				directAccessStart to actually begin a DPA cycle.
 *
 *				If the device requires special hardware setup
 *				for direct access, the handler must assure
 *				that the setup is compatible for the two 
 *				Drawables.
 *				
 *
 *		directAccessDPS
 *				Pointer to a function that will be called
 *				by DisplayPostScript to learn whether DPA is
 *				possible for a given Drawable and also
 *				whether it should use DPA or ddx gc ops to
 *				DPS mark a particular Drawable.
 *
 *				The values that should returned by this
 *				function for various Drawable types should be
 *				determined by the handler developer when
 *				tuning the handler performance.
 *
 *				Tyically the function will return
 *				FALSE if accelerated rendering is possible
 *				for the Drawable (Windows, cached Pixmaps),
 *				and TRUE if the Drawable is a dumb framebuffer.
 *
 *				See the Server Device Developer's Guide for
 *				more information.
 *
 *
 *		directAccessStart	
 *				Pointer to a function that is called to begin
 *				a DPA cycle for a Drawable.
 *
 *				If the handler wishes to allow direct access
 *				to the Drawable's pixels, it should return
 *				TRUE after setting the return parameters, and
 *				setting up any hardware device state
 *				necessary to access the Drawable.
 *
 *				This function will not be called unless
 *				a previous call to the handler's directAccess,
 *				or directAccessDPS function indicated that
 *				DPA is possible for the Drawable.
 *
 *				If an error occurs, directAccessStart should
 *				FALSE.
 *
 *				Any special device state must be maintained
 *				until the direct access cycle is ended
 *				by a call to the handler's directAccessEnd
 *				funtion (if any).
 *
 *				Two DPA cycles may be in progress at the
 *				same time if the handlers directAccess function
 *				returned sunDPABoth.
 *				Also, two cycles might be opened for the same
 *				Drawable.
 *
 *				The return parameters are to be used by the
 *				caller to directly access the Drawable's pixels.
 *
 *				Direct Access should only be allowed if the
 *				Drawable's memory meets the following
 *				requirements.
 *
 *				The return parameters of directAccessStart are
 *
 *				1. a pointer to the beginning of the framebuffer
 *				   for the Drawable, 
 *				2. The number of bytes per row of the Drawable.
 *
 *				The spacing between pixels must be given by the
 *				the following table.
 *
 *				      bitsPerPixel  bytesPerPixel
 *					32	  	4
 *					16	  	2
 *					8	  	1
 *				        4	  	0  (1/2)
 *				        2	  	0  (1/4) (not supported)
 *					1               0  (1/8)
 *
 *				If bytesPerPixel is < 1, the pixels must be
 *				packed in memory, with pixel order matching
 *				the native order for the machine. (This is the
 *				same requirement that the mfb and cfb packages
 *				put on the handler.)
 *				
 *				Pointers to individual pixels are calculated
 *				from these return parameters as follows.
 *
 *				The pointer to the beginning of row y is
 *
 *				    CARD8 *pStart;
 *				    pStart = p + ((y + pDraw->y) * pixelsPerRow)
 *
 *
 *				If bytesPerPixel is >= 1 then the pointer to
 *				the pixel at (x, y) is
 *
 *				pStart + ((x + pDraw->x) * bytesPerPixel)
 *
 *				If bytesPerPixel is 0 (i.e. the Drawable's
 *				depth is less than 8), then the pointer to the
 *				byte containing the pixel is
 *
 *				    pStart + (x + pDraw->x) >> shift
 *
 *				where shift is
 *
 *				    bitsPerPixel      shift
 *				         1         	3
 *				         2         	2
 *				         4         	1
 *
 *		directAccessEnd
 *				Pointer to a function that will be called
 *				inform the handler that Direct Pixel Access
 *				to the Drawable is complete. This function
 *				can be NULL if the handler does not need to
 *				do anything at the end of a direct access
 *				cycle.
 *
 *		reserved[8]	These values are reserved for future use. The
 *				handler should set these to zero before calling
 *				sunDPAScreenInit.
 *
 */

/*
 * Sun DPA Modes
 *
 *	For ease of porting, two of the most common cases
 * 	can be handled with minimal special code in the handler.
 *
 *		sunDPANone
 *				Direct Pixel Access is not allowed to
 *				any Drawable on the handler's Screen.
 *
 *				This is the default mode for a Screen if
 *				the Screen's initialization function.
 *				does not call sunDPAScreenInit().
 *
 *		sunDPACustom
 *				The functions in sunDPAScreenRec are to request
 *				DPA and begin a DPA cycle.
 *
 *		sunDPAPixmap	
 *				Direct mfb,cfb style access is automatic for
 *				all Pixmaps. That is the Pixmaps meet the
 *				following requirements: 
 *
 *				1. devPrivate is a pointer to the Pixmap's
 *				   pixels
 *				2. devKind is the number of bytes per scan line
 *				3. Adjacent pixels are separated by bitsPerPixel
 *				4. Pixel order matches the native bit and byte
 *				   order of the server (same as cfb/mfb).
 *				5. The function pointed to by
 *				   pDraw->pScreen->SourceValidate (if any)
 *				   is the only setup required before accessing
 *				   the Drawable's pixels.
 *
 *		sunDPAAllDrawables
 *				Same as sunDPAPixmap except that access is
 *				allowed for Windows as well. The pointer to
 *				Window's pixels are obtained from the Screen's
 *				"Screen Pixmap", (PixmapPtr)pScreen->devPrivate.
 */

#endif /* !defined(_SUNDPASCR_H) */
