/*
 * Copyright (c) 1992-1995,2000 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#pragma ident "@(#)mpg.h	1.30 00/06/07 SMI"

/*
 *  Multiple plane groups external definitions. 
 */ 

#ifndef MPG_H
#define MPG_H

#include "X.h"
#include "validate.h"
#include "gcstruct.h"
#include "windowstr.h"
#include "scrnintstr.h"
#include "mi/mibstore.h"
#include "servermd.h"

/* max number of plane groups */
#define MPG_MAX_PGS 32

/* attributes for region list preparation */
#define MPG_COLLECT   0x0
#define MPG_COMPUTE   0x1
#define CLIP_BORDER   0x2
#define CLIP_OVERLAY  0x4
#define CLIP_CHILDREN 0x8
#define ONLY_OPAQUE   0x10
#define ONLY_UNDERLAY 0x20
#define ONLY_PHYSICAL 0x40

/* This is a dummy define for spg devices */
#define SPG_PLANE 0

typedef enum {
    MPG_NONE=0, /* no conflict - noop */
    MPG_SAFE,   /* safe conflict - draw, no clip */
    MPG_SOFT,   /* soft conflict - lift, no clip */
    MPG_HARD    /* hard conflict - lift and drop, clip */
} ovlConflict;

typedef enum {
    MPG_NOOP=0,  /* virtual plane group */
    MPG_DRAW, 	 /* drawable by client */
    MPG_FILL, 	 /* fill plane group with constant */
    MPG_FILL_WID /* fill plane group with window id */
} mpgOp;

typedef enum {
    MPG_OTHER=0,	 /* other plane group */
    MPG_VISIBLE,	 /* visible plane group */
    MPG_DRAWABLE,	 /* drawable plane group */
    MPG_VISIBLE_DRAWABLE /* visible drawable plane group */
} mpgType;

typedef enum {
    MPG_PAINT,	 /* drop or draw opaque paint */
    MPG_COVER,	 /* cover previously exposed windows */
    MPG_REMOVE,	 /* lift opaque paint or draw transparent paint */
    MPG_EXPOSE,	 /* expose previously covered windows */
    MPG_REPAIR,	 /* repair damaged windows */
    MPG_OVERLAY  /* mark overlay and its underlays */
} mpgLiftAndDropOp;

typedef RegionPtr *RegionListPtr;
typedef unsigned int mpgPlanes;	 /* bit encoded plane group ids */
typedef unsigned int mpgPlaneId; /* enumerated plane group ids */

typedef struct _mpgRegionList { /* list of regions */
   mpgPlanes planes;
   RegionPtr *pRegions;
} mpgRegionListRec, *mpgRegionListPtr;

typedef struct _mpgWindowList { /* list of overlays or underlays */
   WindowPtr win;
   DevUnion info;		/* extra info for overlay or underlay */
   struct _mpgWindowList *next;
} mpgWindowListRec, *mpgWindowListPtr;

typedef struct _mpgPgInfo {  /* per plane group */
    mpgPlaneId id;           /* external id */
    mpgPlanes plane;         /* bit encoded */
    mpgType type;            /* plane group type */
    mpgOp op;                /* plane group operation */
    DevUnion info;           /* extra info for plane group */
    struct _mpgPgInfo *next; /* next plane group */
} mpgPgInfo, *mpgPgInfoPtr;

/*
 * There are functions that are depth or plane group specific.
 * Some of these functions are stored per visual.
 */
typedef struct _mpgFuncs { 	      /* per visual */
    void (*GetImage)();               /* pDrawable, sx, sy, w, h, format,
                                       * planemask, pDstBits */
    void (*GetSpans)();               /* pDrawable, wmax, pPts, pWidth,
                                       * nspans, pDstBits */
    void (*ResolveColor)();           /* pVisual */
    Bool (*CreateColormap)();         /* pMap */
    void (*DestroyColormap)();        /* pMap */
    void (*CopyWindow)();             /* pWin, oldOrg, pSrcRegion */
    Bool (*CreateWindow)();           /* pWin */
    Bool (*DestroyWindow)();          /* pWin */
    Bool (*RealizeWindow)();          /* pWin */
    Bool (*PositionWindow)();         /* pWin, x, y */
    Bool (*UnrealizeWindow)();        /* pWin */
    void (*PaintWindowBorder)();      /* pWin, pRegion, which */
    void (*PaintWindowBackground)();  /* pWin, pRegion, which */
    Bool (*ChangeWindowAttributes)(); /* pWin, mask */
    miBSFuncPtr BSfuncs;	      /* backing store functions */
    void (*MpgScreenSetup)();         /* pScreen */
} mpgFuncs, *mpgFuncPtr;

#define MPG_PGSwitchMask 		(1L<<0)
#define MPG_CopyPlanesMask		(1L<<1)
#define MPG_AggregatePlanesMask		(1L<<2)
#define MPG_ComputeOpaqueMask		(1L<<3)

typedef struct _mpgScreenFuncs {	/* per screen */
    void (*PGSwitch)();			/* screen switch function */
    void (*CopyPlanes)();		/* plane group copy function */
    int (*AggregatePlanes)();		/* plane group aggregate function */
    RegionPtr (*ComputeOpaque)();	/* overlay opaque shape function */
} mpgScreenFuncs, *mpgScreenFuncPtr;

typedef struct _mpgInfo {   /* per visual */
    mpgPlaneId did;	    /* drawing plane group id */
    mpgPlanes allPlanes;    /* all plane groups for this visual */
    mpgPlanes virtPlanes;   /* virtual plane groups */
    mpgPlanes drawPlanes;   /* drawing + clipping plane groups */
    mpgFuncPtr mpgFuncs;    /* depth and plane group specific functions */
    mpgPgInfoPtr mpgPgList; /* shared plane group list */
    CARD32 whitePixel;	    /* depth specific */
    CARD32 blackPixel;	    /* pixel values */
} mpgInfoRec, *mpgInfoPtr;

typedef struct _mpgPerVisInfo { /* per screen */
    VisualID vid;		
    mpgInfoPtr mpgInfo;		/* vid - mpgInfo pair */
} mpgPerVisInfo, *mpgPerVisInfoPtr;

typedef struct _mpgPerDepthInfo { /* per screen */
    int	depth;
    Bool	(*createGC)();
    PixmapPtr	(*createPixmap)();
    Bool	(*destroyPixmap)();
    void	(*GetImage)();	  /* for pixmaps, since */
    void	(*GetSpans)();	  /* they don't have visuals */
} mpgPerDepthInfo, *mpgPerDepthInfoPtr;

typedef struct _mpgExposed { /* per window */
    mpgPlaneId id;     	     /* plane group id */
    RegionPtr exposed;	     /* exposed region */
    DevUnion info;           /* extra info for exposure */
    struct _mpgExposed *next;
} mpgExposedRec, *mpgExposedPtr;
 
typedef struct _mpgGravity {		 /* gravity */
    mpgPlanes oldPlanes;
    mpgPlanes newPlanes;
    mpgPlanes copyPlanes;
    mpgWindowListPtr pOldOverlays;
    mpgWindowListPtr pNewOverlays;
    RegionPtr pOldRegions[MPG_MAX_PGS];  /* old regions */
    RegionPtr pNewRegions[MPG_MAX_PGS];  /* new regions */
    RegionPtr pCopyRegions[MPG_MAX_PGS]; /* regions to copy */
} mpgGravityRec, *mpgGravityPtr;

typedef struct _mpgClipNode { /* per node */
    RegionPtr clipList;       /* - border - children - overlays clip */
    RegionPtr borderClip;     /* + border + children - overlays clip */
} mpgClipNodeRec, *mpgClipNodePtr;

typedef struct _mpgOvlNode { /* per overlay */
    ovlConflict conflict;    /* drawing conflict */
    DevUnion info;	     /* extra info for overlay */
    DevUnion clip;	     /* extra info for overlay */
} mpgOvlNodeRec, *mpgOvlNodePtr;
 
typedef struct _mpgNode { /* per window */
    mpgPgInfoPtr pginfo;  /* shared plane group info */
    RegionPtr clipList;   /* - border - children clip */
    RegionPtr borderClip; /* + border + children clip */
    DevUnion info;        /* extra info for overlay or underlay */
    struct _mpgNode *next;
} mpgNodeRec, *mpgNodePtr;
 
typedef struct _mpgValidate {	     /* per window */
    Bool ancestor; 	     	     /* affected ancestor */
    union _valUnion { 		     /* validate union */
        struct _before { 	     /* before validate */
    	    DevUnion info; 	     /* extra validation info */
    	    Bool visChanged;         /* visibility state is changed */
            Bool drawCovered;        /* drawable client region is covered */
            Bool drawExposed;        /* drawable client region is exposed */
            Bool drawBorderCovered;  /* drawable border region is covered */
            Bool drawBorderExposed;  /* drawable border region is exposed */
        } before;
        struct _after { 	     /* after validate */
            RegionRec exposed;       /* exposed drawable client region */
            RegionRec borderExposed; /* exposed drawable border region */
        } after;
    } valunion;
    mpgExposedPtr otherExposed;
} mpgValidateRec, *mpgValidatePtr;
 
typedef struct _mpgUnderlay {   /* per underlay */
    mpgPlanes validClipLists;   /* valid clipList plane groups */
    mpgPlanes validBorderClips; /* valid borderClip plane groups */
    mpgWindowListPtr ovlList;   /* list of overlays */
} mpgUnderlayRec, *mpgUnderlayPtr;

typedef struct _mpgOverlay {    /* per overlay */
    RegionPtr ovlClip;          /* overlay clip */
    ovlConflict conflict;       /* drawing conflict */
    Bool opqBorder;             /* opaque border */
    Bool opqBackground;         /* opaque background */
    PixmapPtr ovlMask;	        /* overlay mask */
    Bool ovlMaskValid;	        /* valid overlay mask */
    RegionPtr ovlShape;	        /* overlay shape */
    Bool ovlShapeValid;	        /* valid overlay shape */
    mpgPlanes validClipLists;   /* valid clipList plane groups */
    mpgPlanes validBorderClips; /* valid borderClip plane groups */
    mpgWindowListPtr udlList;   /* list of underlays */
    mpgWindowListPtr ovlList;   /* list of overlays */
} mpgOverlayRec, *mpgOverlayPtr;

typedef struct _mpgPrivGC { /* per gc */
    GCOps *ops;
    GCFuncs *funcs;
} mpgPrivGC, *mpgPrivGCPtr;

typedef struct _mpgPrivWin { /* per window */
    Bool ovlTree;            /* tree has an overlay */
    Bool isOverlay;	     /* window is an overlay */
    RegionPtr oldWinSize;    /* old winSize */
    mpgPlaneId did;	     /* drawing plane group id */
    mpgPlanes treePlanes;    /* tree total plane groups */
    mpgInfoPtr mpgInfo;	     /* shared per visual plane group info */
    mpgValidatePtr valdata;  /* mpg window validation data */
    DevUnion info;	     /* extra info for overlay or underlay */
    mpgNodePtr pglist; 	     /* shared and unique plane group info */
} mpgPrivWin, *mpgPrivWinPtr;

typedef struct _mpgSpgFuncs { /* per spg screen */
    Bool (*RealizeWindow)();          /* pWin */
    Bool (*UnrealizeWindow)();        /* pWin */
    Bool (*PositionWindow)();         /* pWin, x, y */
    void (*WindowExposures)();        /* pWin, pRgn, otherExposed */
    void (*PaintWindowBorder)();      /* pWin, pRegion, which */
    void (*PaintWindowBackground)();  /* pWin, pRegion, which */
    Bool (*ChangeWindowAttributes)(); /* pWin, mask */
    RegionPtr (*TranslateBackingStore)();  /* pWin, windx, windy, pOldClip, oldx, oldy */
    void (*ClearToBackground)();
} mpgSpgFuncsRec, *mpgSpgFuncsPtr;

typedef struct _mpgPrivScreen { /* per screen */
    mpgPlanes dispPlanes;       /* displayable plane groups */
    PixmapPtr pixmap;           /* pointer to screen pixmaps */
    int numPlanegroups;         /* total number of plane groups */
    GCPtr ovlGC;                /* gc for bitmap */
    unsigned long ovlCount;     /* overlay count */
    Bool (*CloseScreen)();      /* screen close function */
    void (*mpgPGSwitch)();      /* screen switch function */
    void (*CopyPlanes)();       /* plane group copy function */
    void (*PreparePlanes)();    /* plane group prepare function */
    int (*AggregatePlanes)();   /* plane group separate/aggregate function */
    mpgPerVisInfoPtr mpgVisInfo;
    mpgPerDepthInfoPtr mpgDepthInfo;
    mpgInfoPtr spgInfo;		/* dummy mpgInfo for spg devices. */
    mpgSpgFuncsPtr spgFuncs;
    int (*ValidateTree)();
    RegionPtr (*ComputeOpaque)();	/* overlay opaque shape function */
} mpgPrivScreen, *mpgPrivScreenPtr;

extern Bool mpgCreateGC();

extern void mpgGetImage();
extern void mpgGetSpans();

extern void mpgChangeInfo();
extern int mpgValidateTree();

extern void mpgResolveColor();
extern Bool mpgCreateColormap();
extern void mpgDestroyColormap();

extern Bool mpgScreenInit();
extern Bool mpgCloseScreen();
extern Bool mpgGetScreenState();
extern Bool mpgInsertPlanegroup();

extern void mpgPrepareWid();
extern void mpgCopyPlanes();
extern void mpgPreparePlanes();
extern void mpgClearToBackground();

extern Bool mpgDestroyPixmap();
extern PixmapPtr mpgCreatePixmap();
extern RegionPtr mpgPixmapToRegion();

extern void mpgCopyWindow();
extern void mpgPaintWindow();
extern Bool mpgCreateWindow();
extern Bool mpgDestroyWindow();
extern Bool mpgRealizeWindow();
extern Bool mpgPositionWindow();
extern Bool mpgUnrealizeWindow();
extern void mpgWindowExposures();
extern Bool mpgChangeWindowAttributes();
	 
extern void mpgSetDrawable();
extern RegionPtr mpgOvlShape();
extern void mpgFillOvlMask();
extern void mpgOvlGrabNotify();
extern DrawablePtr mpgDrawable();
extern void mpgCheckAndDropCursor();
extern mpgInfoPtr getMpgInfoFromVisual();

extern void mpgSaveAreas();
extern void mpgRestoreAreas();
extern void mpgBSExposeCopy();
extern void mpgBSDrawGuarantee();
extern void mpgBSSaveDoomedAreas();
extern RegionPtr mpgBSRestoreAreas();
extern RegionPtr mpgBSClearBackingStore();
extern RegionPtr mpgBSTranslateBackingStore();

extern Bool mpgSetDynaSpg (
#if NeedFunctionPrototypes
   ScreenPtr 		pScreen,
   Bool			on
#endif
);

extern long
mpgSetScreenFuncs(
#if NeedFunctionPrototypes
   ScreenPtr 		pScreen,
   mpgScreenFuncPtr 	funcs,
   long 		mask,
   mpgScreenFuncPtr	oldfuncs
#endif
);

extern int mpg_max_pgs;
extern int mpgGCPrivateIndex;
extern int mpgWindowPrivateIndex;
extern int mpgScreenPrivateIndex;
extern Bool noMinExp; /* no optimized damage */

#define mpg_union(a,b) ((a)|(b)) 
#define mpg_bit_encoded(i) (1<<(i))
#define mpg_intersect(a,b) ((a)&(b))
#define mpg_subtract(a,b) ((a)&(~(b)))
#define mpg_subset(a,b) ((a)==((a)&(b)))

#define mpgUnmapValData ((mpgValidatePtr)1)
#define NullRegionList ((RegionListPtr)NULL)

#define NonPixelParentRelativeBorder(w) (!(w)->borderIsPixel && ((w)->backgroundState == ParentRelative))

#define mpg_priv_scr(pScreen) ((mpgPrivScreenPtr)((pScreen)->devPrivates[mpgScreenPrivateIndex].ptr))

#define mpg_priv_win(pWin) ((mpgPrivWinPtr)((pWin)->devPrivates[mpgWindowPrivateIndex].ptr))

#define mpg_info(pWin) (mpg_priv_win(pWin)->mpgInfo)

#define mpg_valdata(pWin) (mpg_priv_win(pWin)->valdata)
#define mpg_ancestor(pWin) (mpg_valdata(pWin)->ancestor)

#define mpg_drawable_id(pWin) (mpg_priv_win(pWin)->did)
#define mpg_ovl_tree(pWin) (mpg_priv_win(pWin)->ovlTree)
#define mpg_is_overlay(pWin) (mpg_priv_win(pWin)->isOverlay)
#define mpg_old_winsize(pWin) (mpg_priv_win(pWin)->oldWinSize)
#define mpg_toggle(pScreen) (mpg_priv_scr(pScreen)->numPlanegroups == 1 && mpg_priv_scr(pScreen)->spgInfo)

#define mpg_udl(pWin) ((mpgUnderlayPtr)(mpg_priv_win(pWin)->info.ptr))
#define udl_ovl_list(pWin) (mpg_udl(pWin)->ovlList)
#define udl_first_ovl(pWin) (udl_ovl_list(pWin)->win)
#define udl_valid_cliplists(pWin) (mpg_udl(pWin)->validClipLists)
#define udl_valid_borderclips(pWin) (mpg_udl(pWin)->validBorderClips)

#define mpg_ovl(pWin) ((mpgOverlayPtr)(mpg_priv_win(pWin)->info.ptr))
#define mpg_ovl_clip(pWin) (mpg_ovl(pWin)->ovlClip)
#define mpg_ovl_conflict(pWin) (mpg_ovl(pWin)->conflict)
#define mpg_ovl_mask(pWin) (mpg_ovl(pWin)->ovlMask)
#define ovl_mask_valid(pWin) (mpg_ovl(pWin)->ovlMaskValid)
#define ovl_shape(pWin) (mpg_ovl(pWin)->ovlShape)
#define ovl_shape_valid(pWin) (mpg_ovl(pWin)->ovlShapeValid)
#define ovl_ovl_list(pWin) (mpg_ovl(pWin)->ovlList)
#define ovl_first_ovl(pWin) (ovl_ovl_list(pWin)->win)
#define ovl_udl_list(pWin) (mpg_ovl(pWin)->udlList)
#define ovl_first_udl(pWin) (ovl_udl_list(pWin)->win)
#define ovl_opq_border(pWin) (mpg_ovl(pWin)->opqBorder)
#define ovl_opq_background(pWin) (mpg_ovl(pWin)->opqBackground)
#define ovl_valid_cliplists(pWin) (mpg_ovl(pWin)->validClipLists)
#define ovl_valid_borderclips(pWin) (mpg_ovl(pWin)->validBorderClips)

#define udl_is_anc(win) ((win)->info.val)
#define ovl_is_desc(win) ((win)->info.val)
#define ovl_new_state(win) ((win)->info.val)

#define mpg_pg_list(pWin) (mpg_priv_win(pWin)->pglist)
#define mpg_pg_id(pg) ((pg)->pginfo->id)
#define mpg_pg_plane(pg) ((pg)->pginfo->plane)   
#define mpg_pg_type(pg) ((pg)->pginfo->type) 
#define mpg_pg_op(pg) ((pg)->pginfo->op)   
#define mpg_pg_val(pg) ((pg)->pginfo->info.uval) 

#define udl_pg_clip(pg) ((mpgClipNodePtr)((pg)->info.ptr))
#define udl_pg_cliplist(pg) (udl_pg_clip(pg)->clipList)
#define udl_pg_borderclip(pg) (udl_pg_clip(pg)->borderClip)
 
#define ovl_pg(pg) ((mpgOvlNodePtr)((pg)->info.ptr))
#define ovl_pg_conflict(pg) (ovl_pg(pg)->conflict)
#define ovl_pg_val(pg) (ovl_pg(pg)->info.uval)
#define ovl_pg_clip(pg) ((mpgClipNodePtr)(ovl_pg(pg)->clip.ptr))
#define ovl_pg_cliplist(pg) (ovl_pg_clip(pg)->clipList)
#define ovl_pg_borderclip(pg) (ovl_pg_clip(pg)->borderClip)

#define mpg_all_planes(pWin) (mpg_info(pWin)->allPlanes)
#define mpg_virt_planes(pWin) (mpg_info(pWin)->virtPlanes)
#define mpg_draw_planes(pWin) (mpg_info(pWin)->drawPlanes)
#define mpg_tree_planes(pWin) (mpg_priv_win(pWin)->treePlanes)
#define mpg_view_planes(pWin) mpg_subtract(mpg_all_planes(pWin), mpg_draw_planes(pWin))
#define mpg_phys_planes(pWin) mpg_subtract(mpg_all_planes(pWin), mpg_virt_planes(pWin))
#define mpg_vdraw_planes(pWin) mpg_intersect(mpg_draw_planes(pWin), mpg_virt_planes(pWin))
#define mpg_pdraw_planes(pWin) mpg_intersect(mpg_draw_planes(pWin), mpg_phys_planes(pWin))
#define mpg_vview_planes(pWin) mpg_intersect(mpg_view_planes(pWin), mpg_virt_planes(pWin))
#define mpg_pview_planes(pWin) mpg_intersect(mpg_view_planes(pWin), mpg_phys_planes(pWin))

#define mpg_is_clip_pg(pWin, pg)				\
    (mpg_intersect(mpg_pg_plane(pg), mpg_draw_planes(pWin))	\
     || (mpg_is_overlay(pWin) && dgaIsGrabbed(pWin)		\
	 && (mpg_pg_op(pg) == MPG_FILL				\
	     || mpg_pg_op(pg) == MPG_FILL_WID)))


#ifndef NOTDEF
/*
 * For performance and simplification reasons,
 * there's only one visible plane per 
 * window assigned to the first mpgNode.
 */
#define mpg_vis_planes(pWin) ((mpgPlanes)(((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE) || (mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE_DRAWABLE))?mpg_pg_plane(mpg_pg_list(pWin)):0))
#endif /* NOTDEF */
 
#define mpg_vvis_planes(pWin) mpg_intersect(mpg_vis_planes(pWin), mpg_virt_planes(pWin))
#define mpg_pvis_planes(pWin) mpg_intersect(mpg_vis_planes(pWin), mpg_phys_planes(pWin))
#define mpg_vis_cliplist(pWin) ((RegionPtr)(((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE) || (mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE_DRAWABLE))?mpg_pg_list(pWin)->clipList:NULL))
#define mpg_vis_borderclip(pWin) ((RegionPtr)(((mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE) || (mpg_pg_type(mpg_pg_list(pWin)) == MPG_VISIBLE_DRAWABLE))?mpg_pg_list(pWin)->borderClip:NULL))

#define mpg_funcs(pWin) (mpg_info(pWin)->mpgFuncs)
#define mpg_copy_window(pWin, oldOrg, pSrcRegion) (*mpg_funcs(pWin)->CopyWindow)(pWin, oldOrg, pSrcRegion)
#define mpg_create_window(pWin) (* mpg_funcs(pWin)->CreateWindow)(pWin)
#define mpg_destroy_window(pWin) (* mpg_funcs(pWin)->DestroyWindow)(pWin)
#define mpg_realize_window(pWin) (* mpg_funcs(pWin)->RealizeWindow)(pWin)
#define mpg_unrealize_window(pWin) (* mpg_funcs(pWin)->UnrealizeWindow)(pWin)
#define mpg_position_window(pWin, x, y) (* mpg_funcs(pWin)->PositionWindow)(pWin, x, y)
#define mpg_change_window_attributes(pWin, mask) (* mpg_funcs(pWin)->ChangeWindowAttributes)(pWin, mask)
#define mpg_paint_window_border(pWin, pRegion, what) (* mpg_funcs(pWin)->PaintWindowBorder)(pWin, pRegion, what)
#define mpg_paint_window_background(pWin, pRegion, what) (* mpg_funcs(pWin)->PaintWindowBackground)(pWin, pRegion, what)
#define mpg_get_spans(pDrawable, wmax, pPts, pWidth, nspans, pDstBits) (* mpg_funcs((WindowPtr)pDrawable)->GetSpans)(pDrawable, wmax, pPts, pWidth, nspans, pDstBits)
#define mpg_get_image(pDrawable, sx, sy, w, h, format, planemask, pDstBits) (* mpg_funcs((WindowPtr)pDrawable)->GetImage)(pDrawable, sx, sy, w, h, format, planemask, pDstBits)
#define mpg_save_areas(pPixmap, pSave, xorg, yorg, pWin) (* mpg_funcs(pWin)->BSfuncs->SaveAreas)(pPixmap, pSave, xorg, yorg, pWin)
#define mpg_restore_areas(pPixmap, pRestore, xorg, yorg, pWin) (* mpg_funcs(pWin)->BSfuncs->RestoreAreas)(pPixmap, pRestore, xorg, yorg, pWin)
#define mpg_screen_setup(pWin) (* mpg_funcs(pWin)->MpgScreenSetup)((pWin)->drawable.pScreen)

#define mpg_ovl_gc(pScreen) (mpg_priv_scr(pScreen)->ovlGC)
#define mpg_ovl_count(pScreen) (mpg_priv_scr(pScreen)->ovlCount)
#define mpg_disp_planes(pScreen) (mpg_priv_scr(pScreen)->dispPlanes)
#define mpg_num_planes(pScreen) (mpg_priv_scr(pScreen)->numPlanegroups)
#define mpg_prepare_planes(pScreen, pExposed) (* mpg_priv_scr(pScreen)->PreparePlanes)(pScreen, pExposed)
#define mpg_copy_planes(pScreen, pWin, pRegions, planes, dx, dy) (* mpg_priv_scr(pScreen)->CopyPlanes)(pScreen, pWin, pRegions, planes, dx, dy)
#define mpg_aggregate_planes(pScreen, planes) ((int)(mpg_priv_scr(pScreen)->AggregatePlanes?(* mpg_priv_scr(pScreen)->AggregatePlanes)(pScreen, planes):-1))

#ifdef DEBUG
#define MPG_FETCH_OVLSHAPE	FALSE
#define MPG_CALC_OVLSHAPE	TRUE
extern RegionPtr mpgGetOvlShape(WindowPtr, Bool);

#define mpg_ovl_shape(pWin)	mpgGetOvlShape(pWin, MPG_FETCH_OVLSHAPE)
#define mpg_get_ovl_shape(pWin) mpgGetOvlShape(pWin, MPG_CALC_OVLSHAPE)
#else
#define mpg_ovl_shape(pWin)	ovl_shape(pWin)
#define mpg_get_ovl_shape(pWin) (ovl_shape_valid(pWin)?ovl_shape(pWin):mpgOvlShape(pWin))
#endif
#endif /* MPG_H */
