/*
 * Copyright 2004 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/* $XConsortium: sun.h,v 5.38 94/03/28 14:35:05 kaleb Exp $ */

/*-
 * Copyright (c) 1987 by the Regents of the University of California
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */


#ifndef _SUN_H_ 
#define _SUN_H_

#pragma ident	"@(#)sun.h	1.24	04/02/03 SMI"

#if defined(__SVR4) && !defined(SVR4)
#define SVR4
#endif

/* X headers */
#include "Xos.h"
#undef index /* don't mangle silly Sun structure member names */
#include "X.h"
#include "Xproto.h"

/* general system headers */
#ifndef NOSTDHDRS
# include <stdlib.h>
#else
# include <malloc.h>
extern char *getenv();
#endif

/* system headers common to both SunOS and Solaris */
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/filio.h>
#include <sys/ioctl.h>
#include <sys/resource.h>
#ifdef SVR4
#ifdef X_POSIX_C_SOURCE
#define _POSIX_C_SOURCE X_POSIX_C_SOURCE
#include <signal.h>
#undef _POSIX_C_SOURCE
#else
#define _POSIX_SOURCE
#include <signal.h>
#undef _POSIX_SOURCE
#endif
#endif
#include <fcntl.h>
#ifndef i386
# include <poll.h>
#else
# include <sys/poll.h>
#endif
#include <errno.h>
#include <memory.h>

#ifdef X_NOT_STDC_ENV
extern int errno;
#endif

/* 
 * Sun specific headers Sun moved in Solaris.
 *
 * Even if only needed by one source file, I have put them here 
 * to simplify finding them...
 */
#ifdef SVR4
# include <sys/fbio.h>
/* #ifdef sparc */
/* starting in Solaris 8, the following are common to both sparc & intel */
# include <sys/kbd.h>
# include <sys/kbio.h>
# include <sys/msio.h>
/* #endif sparc */
# include <sys/vuid_event.h>
# include <stropts.h>
# define usleep(usec) poll((struct pollfd *) 0, (size_t) 0, usec / 1000)
#else /* SVR4 */
# include <sun/fbio.h>
# include <sundev/kbd.h>
# include <sundev/kbio.h>
# include <sundev/msio.h>
# include <sundev/vuid_event.h>
# include <pixrect/pixrect.h>
# include <pixrect/memreg.h>
extern int ioctl();
extern int getrlimit();
extern int setrlimit();
extern int getpagesize();
#endif /* !SVR4 */
extern int gettimeofday();

/* 
 * Server specific headers
 */
#include "misc.h"
#include "scrnintstr.h"
#include "inputstr.h"
#include "input.h"
#include "colormapst.h"
#include "colormap.h"
#include "cursorstr.h"
#include "cursor.h"
#include "dixstruct.h"
#include "dix.h"
#include "opaque.h"
#include "resource.h"
#include "servermd.h"
#include "windowstr.h"

/* 
 * ddx specific headers 
 */
#ifndef PSZ
#define PSZ 8
#endif

#include "mi/mipointer.h"

#define SHAREDOPENWINTOP "/usr/openwin"
#define LOCALOPENWINTOP "/etc/openwin"

#include    "sunddk.h"

/*
 * MAXEVENTS is the maximum number of events the mouse and keyboard functions
 * will read on a given call to their GetEvents vectors.
 */
#define MAXEVENTS	32

extern int		sunScreenIndex;

/*
 * Frame-buffer-private info.
 *	fd		file opened to the frame buffer device.
 *	info		description of the frame buffer -- type, height, depth,
 *			width, etc.
 *	fb		pointer to the mapped image of the frame buffer. Used
 *			by the driving routines for the specific frame buffer
 *			type.
 *	mapped		flag set true by the driver when the frame buffer has
 *			been mapped in.
 *	parent		set true if the frame buffer is actually a SunWindows
 *			window.
 *	fbPriv		Data private to the frame buffer type.
 */
typedef struct {
    pointer		fb; 	    /* Frame buffer itself */
    Bool		mapped;	    /* TRUE if frame buffer already mapped */
    int			maplength;  /* length of mapping */
    Bool		parent;	    /* TRUE if fd is a SunWindows window */
    int			fd; 	    /* Descriptor open to frame buffer */
    char		devName[128];   /* Device Filename */
    struct fbtype	info;	    /* Frame buffer characteristics */
    void		(*EnterLeave)(ScreenPtr, u_char);    /* screen switch */
    pointer		fbPriv;	    /* Frame-buffer-dependent data */
#if defined (i386) || defined (__i386)
    Bool		(*VtSave)(ScreenPtr);
    Bool		(*VtRestore)(ScreenPtr);
#endif
} fbFd;

typedef struct {
    ColormapPtr		installedMap;
    void		(*WindowExposures)();
    Bool		(*CloseScreen)();
    void		(*UpdateColormap)();
    fbFd		sunFbs;
    Bool		FbUsePwrMgmt;  /* Use PM ioctl on this screen? */
    Bool		(*PwrMgmtCloseScreen)();/* PM close screen wrapper */




} sunScreenRec, *sunScreenPtr;

#define GetScreenPrivate(s) \
    ((sunScreenPtr) ((s)->devPrivates[sunScreenIndex].ptr))
#define SetupScreen(s) sunScreenPtr pPrivate = GetScreenPrivate(s)

extern unsigned long    sunGeneration;

#define GetDevname(index)   sunFbDevList[index].deviceName

/*
 * Cursor functions
 */
extern void 	  sunInitCursor();

/*
 * Initialization
 */
extern Bool sunScreenInit(ScreenPtr);
extern Bool sunSetPixmapFormat(PixmapFormatRec *);
extern void sunCGScreenInit(ScreenPtr);
extern int  sunOpenFrameBuffer(int, struct fbtype *, int, fbFd *, int, char **);

extern volatile int  lastEventTime;    /* Time (in ms.) of last event */
extern void SetTimeSinceLastInputEvent();

extern int monitorResolution;

#define AUTOREPEAT_INITIATE	(500)		/* milliseconds */
#define AUTOREPEAT_DELAY	(50)		/* milliseconds */
/*
 * We signal autorepeat events with the unique Firm_event
 * id AUTOREPEAT_EVENTID.
 * Because inputevent ie_code is set to Firm_event ids in
 * sunKbdProcessEventSunWin, and ie_code is short whereas
 * Firm_event id is u_short, we use 0x7fff.
 */
#define AUTOREPEAT_EVENTID      (0x7fff)        /* AutoRepeat Firm_event id */

#ifdef ENAB_KEYS
extern Bool	tlockActive;
extern long	slowKeyInitiate;
#endif /* ENAB_KEYS */
extern long	autoRepeatInitiate;
extern long	autoRepeatDelay;

extern Bool	noMinExp;

/*-
 * TVTOMILLI(tv)
 *	Given a struct timeval, convert its time into milliseconds...
 */
#define TVTOMILLI(tv)	(((tv).tv_usec/1000)+((tv).tv_sec*1000))

#define MAX_OPTION_LN 25
#define NO_SCREEN -1

#define NO_DEF_VISUAL -1

typedef struct {
    char	*deviceName;
    int		gridx, gridy;
    int		screenLeft;
    int		screenRight;
    int		screenUp;
    int		screenDown;
    int		dpix;
    int		dpiy;
    int		defClass;
    int		defDepth;
    Bool	grayVis;
    char 	*screenConfigName; /* Name of XSCREENCONFIG instance */
} sunFbDevRec, *sunFbDevPtr;

extern sunFbDevRec *sunFbDevList;
extern int	    sunFbDevNum;
#define sunGetDeviceName(index)		sunFbDevList[index].deviceName
#define sunGetScreenConfigName(index)	sunFbDevList[index].screenConfigName

#ifdef XKB
extern Bool             noXkbExtension;
#endif

extern Bool             sunSwapLkeys;











/* These defines are used in specifying the visuals that cfb should export .. */

#define StaticGrayMask  (1 << StaticGray)
#define GrayScaleMask   (1 << GrayScale)
#define StaticColorMask (1 << StaticColor)
#define PseudoColorMask (1 << PseudoColor)
#define TrueColorMask   (1 << TrueColor)
#define DirectColorMask (1 << DirectColor)

#define ALL_VISUALS	(StaticGrayMask  | \
			 GrayScaleMask   | \
			 StaticColorMask | \
			 PseudoColorMask | \
			 TrueColorMask   | \
			 DirectColorMask)

#define GRAY_VISUALS	(StaticGrayMask | \
			 GrayScaleMask)
 
#define NO_VISUALS 0

#define INPUT_DEV_MAJOR 1
#define INPUT_DEV_MINOR 0

/*************************************************************************
 * Some helpful macros for dealing with VUID mouse events.  Moved here so
 * they can be shared with the DDX'es & the scroll wheel code.
 */

/* Limit of number of mouse buttons we can report events on.
 * Currently set to 10 since <sys/vuid_event.h> only defines events for
 * buttons 1-10.  This code assumes BUT(1)-BUT(10) are contigious id's.
 */
#define MAX_BUTTONS	10	

/* Check to see if VUID event code is a mouse button event */
#define IS_VUID_MSBUTTON(e)   ( ((e) >= BUT(1)) && ((e) <= BUT(MAX_BUTTONS)) )

/* Translate VUID mouse button event code to mouse button number */
#define VUID_TO_MSBUTTON(e)	( (e) - BUT(1) + 1 )

/*************************************************************************/

#endif /* _SUN_H_ */
